﻿using System;

namespace HeuristicLab.Problems.ProgramSynthesis.Push.Attributes {
  using HeuristicLab.Problems.ProgramSynthesis.Push.Stack;

  [AttributeUsage(AttributeTargets.Class)]
  public class PushExpressionAttribute : Attribute {
    public readonly StackTypes StackType;
    public readonly StackTypes AdditionalStackDependencies;
    public readonly string Name;
    public readonly string Description;

    /// <summary>
    /// Determines if this expression manipulates the exec stack.
    /// </summary>
    public readonly bool ManipulatesExec;

    /// <summary>
    /// Determines the amount of items fetched form the EXEC stack required for this expression. Used for mapping individuals.
    /// </summary>
    public readonly uint ExecIn;

    /// <summary>
    /// Determines if the expression is visible to the user for configuration purposes.
    /// </summary>
    public readonly bool IsHidden;

    public PushExpressionAttribute(
      StackTypes stackType,
      string name,
      string description,
      StackTypes additionalStackDependencies = default(StackTypes),
      uint execIn = 0,
      bool isHidden = false) {

      StackType = stackType;
      AdditionalStackDependencies = additionalStackDependencies;
      Name = name;
      Description = description;
      ExecIn = execIn;
      IsHidden = isHidden;
      ManipulatesExec = stackType == StackTypes.Exec || AdditionalStackDependencies.HasFlag(StackTypes.Exec);
    }
  }
}
